@@ -43,6 +43,16 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #define DEGREESTORADIANS   M_PI/180.0f
 
 
+/* Modes to interpret coordinates. */
+enum coord_modes
+{
+  MKPROF_MODE_INVALID,          /* For sanity checks.     */
+
+  MKPROF_MODE_IMG,              /* Use image coordinates. */
+  MKPROF_MODE_WCS,              /* Use WCS coordinates.   */
+};
+
+
 
 /* Types of profiles. */
 enum profile_types
@@ -55,6 +65,7 @@ enum profile_types
   PROFILE_POINT,                /* Point profile.              */
   PROFILE_FLAT,                 /* Flat profile.               */
   PROFILE_CIRCUMFERENCE,        /* Circumference profile.      */
+  PROFILE_DISTANCE,             /* Elliptical radius of pixel. */
 
   PROFILE_MAXIMUM_CODE,         /* Just for a sanity check.    */
 };
@@ -81,13 +92,10 @@ struct builtqueue
   size_t               id;    /* ID of this object.                  */
   int               ispsf;    /* This is a PSF profile.              */
   int            overlaps;    /* ==1: Overlaps with the image.       */
-  float              *img;    /* Array of this profile's image.      */
-  size_t         imgwidth;    /* Width of *img.                      */
-  long        fpixel_i[2];    /* First pixel in output image.        */
-  long        lpixel_i[2];    /* Last pixel in output image.         */
-  long        fpixel_o[2];    /* First pixel in this array.          */
+  gal_data_t       *image;    /* Array of this profile's image.      */
+  gal_data_t   *overlap_i;    /* Overlap tile over individual array. */
+  gal_data_t   *overlap_m;    /* Overlap tile over merged array.     */
   int                func;    /* Profile's radial function.          */
-
   int        indivcreated;    /* ==1: an individual file is created. */
   size_t          numaccu;    /* Number of accurate pixels.          */
   double         accufrac;    /* Difference of accurate values.      */
@@ -106,8 +114,9 @@ struct mkprofparams
   char            *backname;  /* Name of background image file name.      */
   char             *catname;  /* Name of catalog of parameters.           */
   char             *backhdu;  /* HDU of background image.                 */
-  long             naxes[2];  /* Size of the output image.                */
+  size_t             *dsize;  /* Size of the output image.                */
   uint8_t       clearcanvas;  /* Pixels in background image set to zero.  */
+  gal_data_t        *kernel;  /* Parameters to define a kernel.           */
   uint8_t        oversample;  /* Oversampling scale.                      */
   uint8_t          psfinimg;  /* ==1: Build PSF profiles in image.        */
   uint8_t        individual;  /* ==1: Build all catalog separately.       */
@@ -116,17 +125,15 @@ struct mkprofparams
   size_t          numrandom;  /* Number of radom points for integration.  */
   float           tolerance;  /* Accuracy to stop integration.            */
   uint8_t          tunitinp;  /* ==1: Truncation is in pixels, not radial.*/
-  long             shift[2];  /* Shift along axeses position of profiles. */
+  size_t             *shift;  /* Shift along axeses position of profiles. */
   uint8_t       prepforconv;  /* Shift and expand by size of first psf.   */
   float           zeropoint;  /* Magnitude of zero point flux.            */
   double        circumwidth;  /* Width of circumference (inward).         */
   uint8_t           replace;  /* Replace overlaping profile pixel values. */
   uint8_t         magatpeak;  /* Mag only for peak pixel, not all profile.*/
   uint8_t           envseed;  /* Use GSL_RNG_SEED for random seed.        */
-  char                *xcol;  /* X column of profile center.              */
-  char                *ycol;  /* Y column of profile center.              */
-  char               *racol;  /* RA column of profile center.             */
-  char              *deccol;  /* Dec column of profile center.            */
+  uint8_t              mode;  /* Coordinates in WCS or image standard.    */
+  gal_list_str_t      *ccol;  /* Columns that keep coordinates.           */
   char                *fcol;  /* Column specifying profile function.      */
   char                *rcol;  /* Effective radius of profile.             */
   char                *ncol;  /* Sersic index column of profile.          */
@@ -135,9 +142,13 @@ struct mkprofparams
   char                *mcol;  /* Magnitude column.                        */
   char                *tcol;  /* Truncation of the profiles.              */
   uint8_t       mforflatpix;  /* mcol is flat pixel value (f is 4 or 5).  */
-  double           crpix[2];  /* CRPIX FITS header keywords.              */
-  double           crval[2];  /* CRVAL FITS header keywords.              */
-  double         resolution;  /* For CDELTi FITS header keywords.         */
+  uint8_t  mcolisbrightness;  /* mcol is total brightness not magnitude.  */
+  gal_data_t         *crpix;  /* CRPIX FITS header keywords.              */
+  gal_data_t         *crval;  /* CRVAL FITS header keywords.              */
+  gal_data_t         *cdelt;  /* For CDELTi FITS header keywords.         */
+  gal_data_t            *pc;  /* WCS PC matrix.                           */
+  gal_data_t         *cunit;  /* Units of each coordinate.                */
+  gal_data_t         *ctype;  /* Type of the coordinates.                 */
 
 
   /* Output */
@@ -150,10 +161,10 @@ struct mkprofparams
   size_t                num;  /* The number of profiles.                  */
   double                 *x;  /* X axis position of profile center.       */
   double                 *y;  /* Y axis position of profile center.       */
-  int                    *f;  /* Profile function code.                   */
+  uint8_t                *f;  /* Profile function code.                   */
   float                  *r;  /* Radius of profile.                       */
   float                  *n;  /* Index of profile.                        */
-  float                  *p;  /* Position angle of profile                */
+  float                  *p;  /* Position angle of profile.               */
   float                  *q;  /* Axis ratio of profile.                   */
   float                  *m;  /* Magnitude of profile.                    */
   float                  *t;  /* Truncation distance.                     */
@@ -168,6 +179,11 @@ struct mkprofparams
   char           *wcsheader;  /* The WCS header information for main img. */
   int            wcsnkeyrec;  /* The number of keywords in the WCS header.*/
   char       *mergedimgname;  /* Name of merged image.                    */
+  int                  nwcs;  /* for WCSLIB: no. coord. representations.  */
+  struct wcsprm        *wcs;  /* WCS information for this dataset.        */
+  size_t               ndim;  /* Number of dimensions (for `nomerged').   */
+                              /* We can't put it in `out' because it is   */
+                              /* meaning ful there.                       */
 };
 
 #endif
