@@ -23,17 +23,33 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #ifndef UI_H
 #define UI_H
 
-void
-checksetfge(char *optarg, int *fge, char *filename, size_t lineno);
+
+
+
+
+/* Available letters for short options:
+
+   a b d e f g j k l m n p r s t u v w x y z
+   A B C E G H J L O Q R W X Y
+*/
+enum option_keys_enum
+{
+  /* With short-option version. */
+  UI_KEY_COLUMN      = 'c',
+  UI_KEY_INFORMATION = 'i',
+
+  /* Only with long version (start with a value 1000, the rest will be set
+     automatically). */
+};
+
+
+
+
 
 void
-checksetfitstabletype(char *optarg, int *fitstabletype, char *filename,
-                   size_t lineno);
+ui_read_check_inputs_setup(int argc, char *argv[], struct tableparams *p);
 
 void
-setparams(int argc, char *argv[], struct tableparams *p);
-
-void
-freeandreport(struct tableparams *p);
+ui_free_report(struct tableparams *p);
 
 #endif
