@@ -92,7 +92,6 @@ gal_fits_io_error(int status, char *message)
 /*************************************************************
  **************           FITS names           ***************
  *************************************************************/
-
 /* IMPORTANT NOTE: if other compression suffixes are add to this function,
    include them in `gal_checkset_automatic_output', so the compression
    suffix can be skipped when the user doesn't specify an output
@@ -102,11 +101,12 @@ gal_fits_name_is_fits(char *name)
 {
   size_t len;
   len=strlen(name);
-  if ( ( len>=4 && strcmp(&name[len-4], "fits") == 0 )
-       || ( len>=7 && strcmp(&name[len-7], "fits.gz") == 0 )
-       || ( len>=6 && strcmp(&name[len-6], "fits.Z") == 0 )
-       || ( len>=3 && strcmp(&name[len-3], "imh") == 0 )
-       || ( len>=7 && strcmp(&name[len-7], "fits.fz") == 0 ) )
+  if ( (    len>=3 && strcmp(&name[len-3], "fit"     ) == 0 )
+       || ( len>=4 && strcmp(&name[len-4], "fits"    ) == 0 )
+       || ( len>=7 && strcmp(&name[len-7], "fits.gz" ) == 0 )
+       || ( len>=6 && strcmp(&name[len-6], "fits.Z"  ) == 0 )
+       || ( len>=3 && strcmp(&name[len-3], "imh"     ) == 0 )
+       || ( len>=7 && strcmp(&name[len-7], "fits.fz" ) == 0 ) )
     return 1;
   else
     return 0;
@@ -123,14 +123,16 @@ gal_fits_name_is_fits(char *name)
 int
 gal_fits_suffix_is_fits(char *suffix)
 {
-  if (strcmp(suffix, "fits") == 0 || strcmp(suffix, ".fits") == 0
-      || strcmp(suffix, "fits.gz") == 0 || strcmp(suffix, ".fits.gz") == 0
-      || strcmp(suffix, "fits.Z") == 0 || strcmp(suffix, ".fits.Z") == 0
-      || strcmp(suffix, "imh") == 0 || strcmp(suffix, ".imh") == 0
-      || strcmp(suffix, "fits.fz") == 0 || strcmp(suffix, ".fits.fz") == 0)
-   return 1;
- else
-   return 0;
+  char *nodot = suffix[0]=='.' ? (suffix+1) : suffix;
+  if ( strcmp(   nodot, "fit"     ) == 0
+       || strcmp(nodot, "fits"    ) == 0
+       || strcmp(nodot, "fits.gz" ) == 0
+       || strcmp(nodot, "fits.Z"  ) == 0
+       || strcmp(nodot, "imh"     ) == 0
+       || strcmp(nodot, "fits.fz" ) == 0 )
+    return 1;
+  else
+    return 0;
 }
 
 
@@ -308,17 +310,22 @@ gal_fits_type_to_datatype(uint8_t type)
       else if( sizeof(int)      == w )   return TINT;
       break;
 
+    /* On 32-bit systems, the length of `int' and `long' are both
+       32-bits. But CFITSIO's LONG type is preferred because it is designed
+       to be 32-bit. Its `INT' type is not clearly defined and caused
+       problems when reading keywords.*/
     case GAL_TYPE_UINT32:
       w=4;
-      if     ( sizeof(int)      == w )   return TUINT;
-      else if( sizeof(long)     == w )   return TULONG;
+      if     ( sizeof(long)     == w )   return TULONG;
+      else if( sizeof(int)      == w )   return TUINT;
       else if( sizeof(short)    == w )   return TUSHORT;
       break;
 
+    /* Similar to UINT32 above. */
     case GAL_TYPE_INT32:
       w=4;
-      if     ( sizeof(int)      == w )   return TINT;
-      else if( sizeof(long)     == w )   return TLONG;
+      if     ( sizeof(long)     == w )   return TLONG;
+      else if( sizeof(int)      == w )   return TINT;
       else if( sizeof(short)    == w )   return TSHORT;
       break;
 
@@ -459,6 +466,86 @@ gal_fits_datatype_to_type(int datatype, int is_table_column)
 
 
 
+/* When there is a BZERO or TZERO and BSCALE or TSCALE keywords, then the
+   type that must be used to store the actual values of the pixels may be
+   different from the type from BITPIX. This function does the necessary
+   corrections.*/
+static void
+fits_type_correct(int *type, double bscale, char *bzero_str)
+{
+  double bzero;
+  int tofloat=1;
+  char *tailptr, *bzero_u64="9223372036854775808";
+
+  /* If bzero_str is given and `bscale=1.0' the case might be that we are
+     dealing with an integer dataset that just needs a different sign. */
+  if(bzero_str && bscale==1.0f)
+    {
+      /* Read the `bzero' string as a `double' number. */
+      bzero  = strtod(bzero_str, &tailptr);
+      if(tailptr==bzero_str)
+        error(EXIT_FAILURE, 0, "%s: BZERO value `%s' couldn't be "
+              "parsed as a number", __func__, bzero_str);
+
+      /* Work based on type. For the default conversions defined by the
+         FITS standard to change the signs of integers, make the proper
+         correction, otherwise set the type to float. */
+      switch(*type)
+        {
+        case GAL_TYPE_UINT8:
+          if(bzero == -128.0f)      { *type = GAL_TYPE_INT8;   tofloat=0; }
+          break;
+
+        case GAL_TYPE_INT16:
+          if(bzero == 32768)        { *type = GAL_TYPE_UINT16; tofloat=0; }
+          break;
+
+        case GAL_TYPE_INT32:
+          if(bzero == 2147483648LU) { *type = GAL_TYPE_UINT32; tofloat=0; }
+          break;
+
+        /* The `bzero' value for unsigned 64-bit integers has 19 decimal
+           digits, but a 64-bit floating point (`double' type) can only
+           safely store 15 decimal digits. As a result, the safest way to
+           check the `bzero' value for this type is to compare it as a
+           string. But all integers nearby (for example
+           `9223372036854775807') get rounded to this same value (when
+           stored as `double'). So we will also check the parsed number and
+           if it equals this number, a warning will be printed. */
+        case GAL_TYPE_INT64:
+          if( !strcmp(bzero_str, bzero_u64) )
+            {*type = GAL_TYPE_UINT64; tofloat=0;}
+          else
+            if( bzero == 9223372036854775808LLU )
+              {
+                fprintf(stderr, "\nWARNING in %s: the BZERO header keyword "
+                        "value (`%s') is very close (but not exactly equal) "
+                        "to `%s'. The latter value in the FITS standard is "
+                        "used to identify that the dataset should be read as "
+                        "unsigned 64-bit integers instead of signed 64-bit "
+                        "integers. Depending on your version of CFITSIO, "
+                        "it may be read as a signed or unsigned 64-bit "
+                        "integer array\n\n", __func__, bzero_str, bzero_u64);
+                tofloat=0;
+              }
+          break;
+
+          /* For the other types (when `BSCALE=1.0f'), currently no
+             correction is necessary, maybe later we can check if the
+             scales are integers and set the integer output type to the
+             smallest type that can allow the scaled values. */
+        default: tofloat=0;
+        }
+    }
+
+  /* If the type must be a float, then do the conversion. */
+  if(tofloat) *type=GAL_TYPE_FLOAT32;
+}
+
+
+
+
+
 
 
 
@@ -588,7 +675,53 @@ gal_fits_hdu_open(char *filename, char *hdu, int iomode)
 
   /* Open the FITS file: */
   if( fits_open_file(&fptr, ffname, iomode, &status) )
-    gal_fits_io_error(status, "reading this FITS file");
+    {
+      switch(status)
+        {
+        /* Since the default HDU is `1', when the file only has one
+           extension, this error is common, so we will put a special
+           notice. */
+        case END_OF_FILE:
+          if( hdu[0]=='1' && hdu[1]=='\0' )
+            error(EXIT_FAILURE, 0, "%s has only one extension/HDU but you "
+                  "have asked for the second HDU (hdu number 1 in CFITSIO)."
+                  "\n\n"
+                  "To fix the problem please add `--hdu=0' (or `-h0') to "
+                  "your command when calling Gnuastro's programs. For "
+                  "library users, please give a value of \"0\" to the HDU "
+                  "argument.\n\n"
+                  "FOOTNOTE -- When writing a new FITS file, Gnuastro leaves "
+                  "the first HDU blank (with no data) and writes the "
+                  "outputs in the second HDU. In this way the keywords of "
+                  "the the first HDU can be used as meta data of the whole "
+                  "file (which may contain many extensions). This is the "
+                  "recommended way in the FITS standard. As a result, "
+                  "Gnuastro's default HDU to read an extension in a FITS "
+                  "file is the second. This error is commonly caused when "
+                  "the FITS file wasn't created according to this "
+                  "convention.", filename);
+          break;
+
+        /* Generic error below is fine for this case */
+        case BAD_HDU_NUM:
+          break;
+
+        /* In case an un-expected error occurs, use the general CFITSIO
+           reporting that we have already prepared. */
+        default:
+          gal_fits_io_error(status, "opening the given extension/HDU in "
+                            "the given file");
+        }
+
+      error(EXIT_FAILURE, 0, "%s: extension/HDU `%s' doesn't exist. Please "
+            "run the following command to see the extensions/HDUs in "
+            "`%s':\n\n"
+            "    $ astfits %s\n\n"
+            "The respective HDU number (or name, when present) may be used "
+            "with the `--hdu' option in Gnuastro's programs (or the `hdu' "
+            "argument in Gnuastro's libraries) to open the respective HDU.",
+            filename, hdu, filename, filename);
+    }
 
   /* Clean up and the pointer. */
   free(ffname);
@@ -1036,7 +1169,6 @@ gal_fits_key_write_wcsstr(fitsfile *fptr, char *wcsstr, int nkeyrec)
 {
   size_t i;
   int h, status=0;
-  char startblank[]="                      / ";
   char *cp, *cpf, blankrec[80], titlerec[80];
 
   /* Set the last element of the blank array. */
@@ -1048,7 +1180,7 @@ gal_fits_key_write_wcsstr(fitsfile *fptr, char *wcsstr, int nkeyrec)
   /* Print the first two lines before the WCS header information. */
   if(fits_write_record(fptr, blankrec, &status))
     gal_fits_io_error(status, NULL);
-  sprintf(titlerec, "%sWCS information", startblank);
+  sprintf(titlerec, "%sWCS information", GAL_FITS_KEY_TITLE_START);
   for(i=strlen(titlerec);i<79;++i)
     titlerec[i]=' ';
   if(fits_write_record(fptr, titlerec, &status))
@@ -1123,7 +1255,6 @@ gal_fits_key_write_version(fitsfile *fptr, gal_fits_list_key_t *headers,
   int status=0;
   char *gitdescribe;
   char cfitsioversion[20];
-  char startblank[]="              / ";
   char *cp, *cpf, blankrec[80], titlerec[80];
 
   /* Before WCSLIB 5.0, the wcslib_version function was not
@@ -1146,7 +1277,7 @@ gal_fits_key_write_version(fitsfile *fptr, gal_fits_list_key_t *headers,
   if(headers)
     {
       fits_write_record(fptr, blankrec, &status);
-      sprintf(titlerec, "%s%s", startblank,
+      sprintf(titlerec, "%s%s", GAL_FITS_KEY_TITLE_START,
               program_name ? program_name : PACKAGE_NAME);
       for(i=strlen(titlerec);i<79;++i) titlerec[i]=' ';
       fits_write_record(fptr, titlerec, &status);
@@ -1156,7 +1287,7 @@ gal_fits_key_write_version(fitsfile *fptr, gal_fits_list_key_t *headers,
 
   /* Start printing the version information */
   fits_write_record(fptr, blankrec, &status);
-  sprintf(titlerec, "%sVersions and date", startblank);
+  sprintf(titlerec, "%sVersions and date", GAL_FITS_KEY_TITLE_START);
   for(i=strlen(titlerec);i<79;++i) titlerec[i]=' ';
   fits_write_record(fptr, titlerec, &status);
   gal_fits_io_error(status, NULL);
@@ -1226,14 +1357,19 @@ gal_fits_key_write_version(fitsfile *fptr, gal_fits_list_key_t *headers,
 
 /* Note that the FITS standard defines any array as an `image',
    irrespective of how many dimensions it has. This function will return
-   the Gnuastro-type and also the number of dimensions and size along each
-   dimension of the image. Note that `*dsize' will be allocated here, so it
-   must not point to any already allocated space. */
+   the Gnuastro-type, the number of dimensions and size along each
+   dimension of the image along with its name and units if necessary (not
+   NULL). Note that `*dsize' will be allocated here, so it must not point
+   to any already allocated space. */
 void
-gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize)
+gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
+                  char **name, char **unit)
 {
-  size_t i;
+  double bscale=NAN;
+  size_t i, dsize_key=1;
+  char **str, *bzero_str=NULL;
   int bitpix, status=0, naxis;
+  gal_data_t *key, *keysll=NULL;
   long naxes[GAL_FITS_MAX_NDIM];
 
   /* Get the BITPIX, number of dimensions and size of each dimension. */
@@ -1245,11 +1381,57 @@ gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize)
   /* Convert bitpix to Gnuastro's known types. */
   *type=gal_fits_bitpix_to_type(bitpix);
 
+  /* Define the names of the possibly existing important keywords about the
+     dataset. We are defining these in the opposite order to be read by
+     CFITSIO. The way Gnuastro writes the FITS keywords, the output will
+     first have `BZERO', then `BSCALE', then `EXTNAME', then, `BUNIT'.*/
+  gal_list_data_add_alloc(&keysll, NULL, GAL_TYPE_STRING, 1, &dsize_key,
+                          NULL, 0, -1, "BUNIT", NULL, NULL);
+  gal_list_data_add_alloc(&keysll, NULL, GAL_TYPE_STRING, 1, &dsize_key,
+                          NULL, 0, -1, "EXTNAME", NULL, NULL);
+  gal_list_data_add_alloc(&keysll, NULL, GAL_TYPE_FLOAT64, 1, &dsize_key,
+                          NULL, 0, -1, "BSCALE", NULL, NULL);
+  gal_list_data_add_alloc(&keysll, NULL, GAL_TYPE_STRING, 1, &dsize_key,
+                          NULL, 0, -1, "BZERO", NULL, NULL);
+  gal_fits_key_read_from_ptr(fptr, keysll, 0, 0);
+
+
+  /* Read the special keywords. */
+  i=1;
+  for(key=keysll;key!=NULL;key=key->next)
+    {
+      /* Recall that the order is the opposite (this is a last-in-first-out
+         list. */
+      if(key->status==0)
+        {
+        switch(i)
+          {
+          case 4: if(unit) {str = key->array; *unit = *str; *str=NULL;} break;
+          case 3: if(name) {str = key->array; *name = *str; *str=NULL;} break;
+          case 2: bscale = *(double *)(key->array);                     break;
+          case 1: str = key->array; bzero_str = *str;                   break;
+          default:
+            error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+                  "fix the problem. For some reason, there are more "
+                  "keywords requested ", __func__, PACKAGE_BUGREPORT);
+          }
+        }
+      ++i;
+    }
+
+  if( !isnan(bscale) || bzero_str )
+    fits_type_correct(type, bscale, bzero_str);
+
+
   /* Allocate the array to keep the dimension size and fill it in, note
      that its order is the opposite of naxes. */
   *dsize=gal_data_malloc_array(GAL_TYPE_INT64, *ndim, __func__, "dsize");
   for(i=0; i<*ndim; ++i)
     (*dsize)[i]=naxes[*ndim-1-i];
+
+
+  /* Clean up. */
+  gal_list_data_free(keysll);
 }
 
 
@@ -1261,13 +1443,12 @@ gal_data_t *
 gal_fits_img_read(char *filename, char *hdu, size_t minmapsize)
 {
   void *blank;
-  int anyblank;
   long *fpixel;
   fitsfile *fptr;
-  int status=0, type;
-  gal_data_t *img, *keysll=NULL;
-  char **str, *name=NULL, *unit=NULL;
-  size_t i, ndim, *dsize, dsize_key=1;
+  gal_data_t *img;
+  size_t i, ndim, *dsize;
+  char *name=NULL, *unit=NULL;
+  int status=0, type, anyblank;
 
 
   /* Check HDU for realistic conditions: */
@@ -1275,7 +1456,7 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize)
 
 
   /* Get the info and allocate the data structure. */
-  gal_fits_img_info(fptr, &type, &ndim, &dsize);
+  gal_fits_img_info(fptr, &type, &ndim, &dsize, &name, &unit);
 
 
   /* Check if there is any dimensions (the first header can sometimes have
@@ -1290,30 +1471,20 @@ gal_fits_img_read(char *filename, char *hdu, size_t minmapsize)
           hdu);
 
 
-  /* Set the fpixel array (first pixel in all dimensions): */
+  /* Set the fpixel array (first pixel in all dimensions). Note that the
+     `long' type will not be larger than 64-bits, so, we'll just assume it
+     is 64-bits for space allocation. On 32-bit systems, this won't be a
+     problem, the space will be written/read as 32-bit `long' any way,
+     we'll just have a few empty bytes that will be freed anyway at the end
+     of this function. */
   fpixel=gal_data_malloc_array(GAL_TYPE_INT64, ndim, __func__, "fpixel");
   for(i=0;i<ndim;++i) fpixel[i]=1;
 
 
-  /* Read the possibly existing useful keywords. Note that the values are
-     in allocated strings in the keys[i] data structures. Note that we need
-     the linked list of keys to keep the `name' and `unit' pointers. We can
-     free the linked list after `gal_data_alloc' has read/copied the
-     values.*/
-  gal_list_data_add_alloc(&keysll, NULL, GAL_TYPE_STRING, 1, &dsize_key,
-                          NULL, 0, -1, "EXTNAME", NULL, NULL);
-  gal_list_data_add_alloc(&keysll, NULL, GAL_TYPE_STRING, 1, &dsize_key,
-                          NULL, 0, -1, "BUNIT", NULL, NULL);
-  gal_fits_key_read_from_ptr(fptr, keysll, 0, 0);
-  if(keysll->status==0)       {str=keysll->array;       unit=*str; }
-  if(keysll->next->status==0) {str=keysll->next->array; name=*str; }
-
-
   /* Allocate the space for the array and for the blank values. */
   img=gal_data_alloc(NULL, type, ndim, dsize, NULL, 0, minmapsize,
                      name, unit, NULL);
   blank=gal_blank_alloc_write(type);
-  gal_list_data_free(keysll);
   free(dsize);
 
 
@@ -1426,32 +1597,97 @@ fitsfile *
 gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
 {
   void *blank;
-  char *wcsstr;
+  int64_t *i64;
   fitsfile *fptr;
+  uint64_t *u64, *u64f;
   long fpixel=1, *naxes;
   size_t i, ndim=input->ndim;
-  gal_data_t *towrite, *block=gal_tile_block(input);
-  int nkeyrec, status=0, datatype=gal_fits_type_to_datatype(block->type);
+  int nkeyrec, hasblank, status=0, datatype=0;
+  char *wcsstr, *u64key;
+  gal_data_t *i64data, *towrite, *block=gal_tile_block(input);
 
   /* If the input is a tile (isn't a contiguous region of memory), then
      copy it into a contiguous region. */
   towrite = input==block ? input : gal_data_copy(input);
+  hasblank=gal_blank_present(towrite, 0);
 
   /* Allocate the naxis area. */
   naxes=gal_data_malloc_array( ( sizeof(long)==8
                                  ? GAL_TYPE_INT64
                                  : GAL_TYPE_INT32 ), ndim, __func__, "naxes");
 
+
   /* Open the file for writing */
   fptr=gal_fits_open_to_write(filename);
 
+
   /* Fill the `naxes' array (in opposite order, and `long' type): */
   for(i=0;i<ndim;++i) naxes[ndim-1-i]=towrite->dsize[i];
 
-  /* Create the FITS file. */
-  fits_create_img(fptr, gal_fits_type_to_bitpix(towrite->type),
-                  ndim, naxes, &status);
-  gal_fits_io_error(status, NULL);
+
+  /* Create the FITS file. Unfortunately CFITSIO doesn't have a macro for
+     UINT64, TLONGLONG is only for (signed) INT64. So if the dataset has
+     that type, we'll have to convert it to `INT64' and in the mean-time
+     shift its zero, we will then have to write the BZERO and BSCALE
+     keywords accordingly. */
+  if(block->type==GAL_TYPE_UINT64)
+    {
+      /* Allocate the necessary space. */
+      i64data=gal_data_alloc(NULL, GAL_TYPE_INT64, ndim, towrite->dsize,
+                             NULL, 0, block->minmapsize, NULL, NULL, NULL);
+
+      /* Copy the values while making the conversion. */
+      i64=i64data->array;
+      u64f=(u64=towrite->array)+towrite->size;
+      if(hasblank)
+        {
+          do *i64++ = ( *u64==GAL_BLANK_UINT64
+                        ? GAL_BLANK_INT64
+                        : (*u64 + INT64_MIN) );
+          while(++u64<u64f);
+        }
+      else
+        do *i64++ = (*u64 + INT64_MIN); while(++u64<u64f);
+
+      /* We can now use CFITSIO's signed-int64 type macros. */
+      datatype=TLONGLONG;
+      fits_create_img(fptr, LONGLONG_IMG, ndim, naxes, &status);
+      gal_fits_io_error(status, NULL);
+
+
+      /* Write the image into the file. */
+      fits_write_img(fptr, datatype, fpixel, i64data->size, i64data->array,
+                     &status);
+      gal_fits_io_error(status, NULL);
+
+
+      /* We need to write the BZERO and BSCALE keywords manually. VERY
+         IMPORTANT: this has to be done after writing the array. We cannot
+         write this huge integer as a variable, so we'll simply write the
+         full record/card. It is just important that the string be larger
+         than 80 characters, CFITSIO will trim the rest of the string. */
+      u64key="BZERO   =  9223372036854775808 / Offset of data                                         ";
+      fits_write_record(fptr, u64key, &status);
+      u64key="BSCALE  =                    1 / Default scaling factor                                 ";
+      fits_write_record(fptr, u64key, &status);
+      gal_fits_io_error(status, NULL);
+    }
+  else
+    {
+      /* Set the datatype */
+      datatype=gal_fits_type_to_datatype(block->type);
+
+      /* Create the FITS file. */
+      fits_create_img(fptr, gal_fits_type_to_bitpix(towrite->type),
+                      ndim, naxes, &status);
+      gal_fits_io_error(status, NULL);
+
+      /* Write the image into the file. */
+      fits_write_img(fptr, datatype, fpixel, towrite->size, towrite->array,
+                     &status);
+      gal_fits_io_error(status, NULL);
+    }
+
 
   /* Remove the two comment lines put by CFITSIO. Note that in some cases,
      it might not exist. When this happens, the status value will be
@@ -1461,13 +1697,10 @@ gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
   fits_delete_key(fptr, "COMMENT", &status);
   status=0;
 
-  /* Write the image into the file. */
-  fits_write_img(fptr, datatype, fpixel, towrite->size, towrite->array,
-                 &status);
 
   /* If we have blank pixels, we need to define a BLANK keyword when we are
      dealing with integer types. */
-  if(gal_blank_present(towrite, 0))
+  if(hasblank)
     switch(towrite->type)
       {
       case GAL_TYPE_FLOAT32:
@@ -1484,14 +1717,17 @@ gal_fits_img_write_to_ptr(gal_data_t *input, char *filename)
         free(blank);
       }
 
+
   /* Write the extension name to the header. */
   if(towrite->name)
     fits_write_key(fptr, TSTRING, "EXTNAME", towrite->name, "", &status);
 
+
   /* Write the units to the header. */
   if(towrite->unit)
     fits_write_key(fptr, TSTRING, "BUNIT", towrite->unit, "", &status);
 
+
   /* Write comments if they exist. */
   if(towrite->comment)
     fits_write_comment(fptr, towrite->comment, &status);
@@ -1603,10 +1839,10 @@ gal_fits_img_write_corr_wcs_str(gal_data_t *input, char *filename,
      original version, we just want to change the copied version. */
   if(crpix)
     {
-      fits_update_key(fptr, TDOUBLE, "CRPIX1", &crpix[0],
-                      NULL, &status);
-      fits_update_key(fptr, TDOUBLE, "CRPIX2", &crpix[1],
-                      NULL, &status);
+      fits_update_key(fptr, TDOUBLE, "CRPIX1", &crpix[0], NULL, &status);
+      fits_update_key(fptr, TDOUBLE, "CRPIX2", &crpix[1], NULL, &status);
+      if(input->ndim==3)
+        fits_update_key(fptr, TDOUBLE, "CRPIX3", &crpix[2], NULL, &status);
       gal_fits_io_error(status, NULL);
     }
 
@@ -2381,7 +2617,7 @@ gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
 
 
   /* Remove the output if it already exists. */
-  gal_checkset_check_remove_file(filename, 0, dontdelete);
+  gal_checkset_writable_remove(filename, 0, dontdelete);
 
 
   /* Create the FITS file */
